@@ -1,3 +1,4 @@
 from .core import connect, sync, status_file, upload_file, download_file, get_credentials, check_changes, \
-    remove_sync_status, get_bucket_keys, read_yaml, read_json, write_json, write_yaml, Operation, Location
+    remove_sync_status, get_bucket_keys, read_yaml, read_json, write_json, write_yaml, exists, split, \
+    Operation, Location
 from .monitor import Monitor
