@@ -1,22 +1,20 @@
 import os
+import hashlib
 
 import boto3
-import hashlib
 from typing import Dict, Tuple, Set, Any, List
 from botocore.client import BaseClient
-from os import environ, makedirs, utime
-from os.path import expanduser, join, exists, dirname, splitext, realpath
+from os.path import expanduser, join, dirname, splitext, realpath
 
 from botocore.exceptions import ClientError
 from mysutils.tmp import removable_tmp
 from mysutils.yaml import load_yaml, save_yaml
 from mysutils.file import save_json, load_json
-from mysutils.hash import file_md5
 from dateutil.tz import tzlocal
 from logging import getLogger
 from enum import Enum
 
-from filecloudsync.file import get_folder_files
+from filecloudsync.file import key_to_path, file_etag, get_folder_files
 from filecloudsync.s3.exceptions import TagsNotMatchError, S3ConnectionError
 
 logger = getLogger(__name__)
@@ -52,14 +50,17 @@ class Location(Enum):
     BUCKET = "keys"
 
 
-def key_to_path(folder: str, key: str) -> str:
-    """ Convert a key into a file path
+def status_file(endpoint: str, bucket: str, folder: str, location: Location) -> str:
+    """ Get the file path of the synchronization status file
 
-    :param folder: The folder from which the key is synchronized
-    :param key: The key to convert
-    :return: A path to file in the folder which represents that key
+    :param endpoint: The S3 endpoint
+    :param bucket: The bucket name
+    :param folder: The folder to synchronize with
+    :param location: If the cache is for local files or for bucket keys
+    :return: The file path to the status cache file
     """
-    return join(folder, *key.split('/'))
+    hash_code = hashlib.sha1(''.join([endpoint, bucket, realpath(folder)]).encode('utf-8')).hexdigest()
+    return join(_status_folder(), f'{hash_code}.{location.value}.json.gz')
 
 
 def get_credentials(**kwargs) -> dict:
@@ -73,13 +74,13 @@ def get_credentials(**kwargs) -> dict:
     :param kwargs: The possible S3 credentials
     :return: A dictionary with the detected credentials
     """
-    access_key = kwargs.get('aws_access_key_id', environ.get(ACCESS_KEY_ENV))
-    secret_key = kwargs.get('aws_secret_access_key', environ.get(SECRET_KEY_ENV))
-    endpoint = kwargs.get('endpoint_url', environ.get(ENDPOINT_ENV))
+    access_key = kwargs.get('aws_access_key_id', os.environ.get(ACCESS_KEY_ENV))
+    secret_key = kwargs.get('aws_secret_access_key', os.environ.get(SECRET_KEY_ENV))
+    endpoint = kwargs.get('endpoint_url', os.environ.get(ENDPOINT_ENV))
     for file in [S3_CONF_FILE, CONFIG_PATH]:
-        if exists(file):
+        if os.path.exists(file):
             s3conf = load_yaml(file)
-            for key, value in kwargs:
+            for key, value in kwargs.items():
                 if key == 'aws_access_key_id':
                     s3conf[key] = access_key if access_key else s3conf.get(key)
                 elif key == 'aws_secret_access_key':
@@ -123,25 +124,25 @@ def upload_file(file: str, client: BaseClient, bucket: str, key: str) -> (float,
     client.upload_file(file, bucket, key)
     bucket_object = client.head_object(Bucket=bucket, Key=key)
     last_modified = bucket_object['LastModified'].astimezone(tzlocal()).timestamp()
-    utime(file, (last_modified, last_modified))
+    os.utime(file, (last_modified, last_modified))
     return last_modified, bucket_object['ETag'].replace('"', '')
 
 
 def download_file(client: BaseClient, bucket: str, key: str, dest: str) -> (float, str):
-    """
+    """ Download a file form a S3 bucket.
 
-    :param client:
-    :param bucket:
-    :param key:
-    :param dest:
-    :return:
+    :param client: The client
+    :param bucket: The bucket name.
+    :param key: The key of the bucket file
+    :param dest: Where the key is stored in
+    :return: The last modified date and the key eTag
     """
     file_path = key_to_path(dest, key)
-    makedirs(dirname(file_path), exist_ok=True)
+    os.makedirs(dirname(file_path), exist_ok=True)
     client.download_file(bucket, key, file_path)
     bucket_object = client.head_object(Bucket=bucket, Key=key)
     last_modified = bucket_object['LastModified'].astimezone(tzlocal()).timestamp()
-    utime(file_path, (last_modified, last_modified))
+    os.utime(file_path, (last_modified, last_modified))
     return last_modified, bucket_object['ETag'].replace('"', '')
 
 
@@ -153,19 +154,6 @@ def _status_folder() -> str:
     return join(expanduser('~'), S3_STATUS_FOLDER)
 
 
-def status_file(endpoint: str, bucket: str, folder: str, location: Location) -> str:
-    """ Get the file path of the synchronization status file
-
-    :param endpoint: The S3 endpoint
-    :param bucket: The bucket name
-    :param folder: The folder to synchronize with
-    :param location: If the cache is for local files or for bucket keys
-    :return: The file path to the status cache file
-    """
-    hash_code = hashlib.sha1(''.join([endpoint, bucket, realpath(folder)]).encode('utf-8')).hexdigest()
-    return join(_status_folder(), f'{hash_code}.{location.value}.json.gz')
-
-
 def _save_sync_status(
         endpoint: str,
         bucket: str,
@@ -182,7 +170,7 @@ def _save_sync_status(
     :param local_files: The list of local files to synchronize
     """
     status_folder = _status_folder()
-    makedirs(status_folder, exist_ok=True)
+    os.makedirs(status_folder, exist_ok=True)
     keys = {key: last_modified for key, last_modified in bucket_files.items()}
     files = {file: last_modified for file, last_modified in local_files.items()}
     save_json(keys, status_file(endpoint, bucket, folder, Location.BUCKET))
@@ -260,7 +248,7 @@ def _load_sync_status(
     :return: The cached bucket keys and the cached local files with their timestamps and hash
     """
     status_file_path = status_file(endpoint, bucket, folder, where)
-    status = load_json(status_file_path) if exists(status_file_path) else {}
+    status = load_json(status_file_path) if os.path.exists(status_file_path) else {}
     return {key: value for key, value in status.items() if not files or key in files}
 
 
@@ -271,9 +259,9 @@ def remove_sync_status(endpoint: str, bucket: str, folder: str) -> None:
     :param bucket: The bucket name
     :param folder: The folder to synchronize with
     """
-    if folder and exists(status_file(endpoint, bucket, folder, Location.BUCKET)):
+    if folder and os.path.exists(status_file(endpoint, bucket, folder, Location.BUCKET)):
         os.remove(status_file(endpoint, bucket, folder, Location.BUCKET))
-    if folder and exists(status_file(endpoint, bucket, folder, Location.LOCAL)):
+    if folder and os.path.exists(status_file(endpoint, bucket, folder, Location.LOCAL)):
         os.remove(status_file(endpoint, bucket, folder, Location.LOCAL))
 
 
@@ -417,7 +405,7 @@ def _download_file(
     :return: A tuple with the bucket key timestamp and hash and the local file timestamp and hash
     """
     bucket_last_modified, bucket_etag = download_file(client, bucket, key, folder)
-    local_last_modified, local_etag = bucket_last_modified, file_md5(key_to_path(folder, key))
+    local_last_modified, local_etag = bucket_last_modified, file_etag(key_to_path(folder, key))
     if bucket_etag != local_etag:
         raise TagsNotMatchError(f'The local tag {local_etag} differs to bucket one {bucket_etag}.')
     bucket_files[key], local_files[key] = (bucket_last_modified, bucket_etag), (bucket_last_modified, bucket_etag)
@@ -543,7 +531,13 @@ def apply_local_changes(
             _sync_bucket_remove(client, bucket, key, folder, bucket_files, local_files)
 
 
-def sync(client: BaseClient, bucket: str, folder: str, files: Set[str] = None) -> List[Tuple[str, Operation, Location]]:
+def sync(
+        client: BaseClient,
+        bucket: str,
+        folder: str,
+        files: Set[str] = None,
+        readonly: bool = False
+) -> List[Tuple[str, Operation, Location]]:
     """ Synchronize a S3 bucket and a folder.
 
     :param client: The S3 client.
@@ -551,8 +545,9 @@ def sync(client: BaseClient, bucket: str, folder: str, files: Set[str] = None) -
     :param folder: The folder to synchronize with.
     :param files: A list of keys to watch in Unix file path format.
             If none is given, then check all the bucket/folder files.
+    :param readonly: If the bucket is a readonly bucket.
     """
-    bucket_diff, bucket_files, local_diff, local_files = check_changes(client, bucket, folder, files)
+    bucket_diff, bucket_files, local_diff, local_files = check_changes(client, bucket, folder, files, readonly)
     try:
         # Apply changes
         apply_bucket_changes(client, bucket, folder, bucket_files, local_files, bucket_diff)
@@ -568,7 +563,8 @@ def check_changes(
         client: BaseClient,
         bucket: str,
         folder: str,
-        files: Set[str] = None
+        files: Set[str] = None,
+        readonly: bool = False
 ) -> Tuple[Dict[str, Operation], Dict[str, Tuple[float, str]], Dict[str, Operation], Dict[str, Tuple[float, str]]]:
     """ Check the changes of a S3 bucket and a folder.
 
@@ -577,14 +573,41 @@ def check_changes(
     :param folder: The folder to check.
     :param files: A list of keys to watch in Unix file path format.
             If none is given, then check all the bucket/folder files.
-
+    :param readonly: If the bucket is a readonly bucket.
     :return: A tuple with the bucket differences, the bucket files, the local differences and the local files.
     """
     bucket_diff, bucket_files = check_bucket_changes(client, bucket, folder, files)
     local_diff, local_files = check_local_changes(client, bucket, folder, files)
+    local_diff = {} if readonly else local_diff
+    _remove_exactly_equal_files(bucket_diff, bucket_files, local_diff, local_files)
     return bucket_diff, bucket_files, local_diff, local_files
 
 
+def _remove_exactly_equal_files(
+        bucket_diff: Dict[str, Operation],
+        bucket_files: Dict[str, tuple[float, str]],
+        local_diff: Dict[str, Operation],
+        local_files: Dict[str, tuple[float, str]]
+) -> None:
+    """ Remove the supposed changes that actually are the same files.
+        It can happend when the synchronization cache is removed.
+
+    :param bucket_diff: The list of changes detected in the bucket
+    :param bucket_files: The list of bucket files
+    :param local_diff: The list of changes detected in the local folder
+    :param local_files: The list of folder files
+    """
+    added_bucket_operations = {key for key, op in bucket_diff.items() if op == Operation.ADDED}
+    added_local_operations = {key for key, op in local_diff.items() if op == Operation.ADDED}
+    for key in added_bucket_operations | added_local_operations:
+        if key in bucket_files and key in local_files and \
+                (bucket_files[key][0] != local_files[key][0] or int(bucket_files[key][1]) != local_files[key][1]):
+            if key in bucket_diff:
+                del bucket_diff[key]
+            if key in local_diff:
+                del local_diff[key]
+
+
 def check_bucket_changes(
         client: BaseClient,
         bucket: str,
@@ -705,3 +728,38 @@ def write_yaml(obj: Any, client: BaseClient, bucket: str, key: str, encoding: st
     with removable_tmp(suffix=splitext(key)[1]) as tmp_file:
         save_yaml(obj, tmp_file, True, encoding)
         client.upload_file(tmp_file, bucket, key)
+
+
+def exists(client: BaseClient, bucket: str, *keys: str) -> bool:
+    """ Check if a bucket and a key exist.
+
+    :param client: The s3 client
+    :param bucket: The s3 bucket
+    :param key: The bucket key. If it is not given, then it only check the bucket.
+    :return: True if that bucket and key exist if a key is given, if not, True if that bucket exists, otherwise False
+    """
+    try:
+        # Send a request to check the bucket
+        for key in keys:
+            client.head_bucket(Bucket=bucket)
+            if key:
+                client.head_object(Bucket=bucket, Key=key)
+        return True
+    except ClientError:
+        # If an error occurs, the bucket does not exist or is not accessible
+        return False
+
+
+def split(uri: str) -> Tuple[str, str, str]:
+    """ Split a URL to a bucket key to protocol, bucket name and path
+
+    :param uri: The URI to the key
+    :return: A tuple with the protocol (typically s3://), the bucket name and the key path
+    """
+    uri = uri.strip()
+    protocol = 's3://' if uri.startswith('s3://') else ''
+    uri = uri[len(protocol):]
+    pos = uri.find('/')
+    name = uri if pos == -1 else uri[:pos]
+    path = '' if pos == -1 else uri[pos + 1:]
+    return protocol, name, path
